/*
Stallation version 0.7  (2000-06-28)

Written by Peter Folk <pfolk@uni.uiuc.edu>

The contents of this file are in the public domain, but
the author would appreciate it if any useful changes or
corrections were communicated back to him, for inclusion
in a future version.
*/
#include "InstallationManager.h"
#include <be/interface/Alert.h>
#include <be/support/String.h>
#include <layout/VGroup.h>
#include <layout/HGroup.h>
#include <layout/MBorder.h>
#include <layout/MScrollView.h>
#include <layout/MStringView.h>
#include <layout/MListView.h>
#include <layout/Space.h>
#include <layout/MButton.h>
#include <image.h>  /* load_image() */ 
#include <OS.h>     /* resume_thread() */ 
#include <stdlib.h> /* environ */ 
#include <be/storage/File.h>
#include <be/app/Application.h>

#define ITEMS_INVOKED 'InVo'

InstallationManager::InstallationManager(BRect size)
  :MWindow(size,"Installation Manager",B_TITLED_WINDOW,0),
   list(new MListView(B_MULTIPLE_SELECTION_LIST))
{
	// Fill list
	Stallation* item;
	for (int i=0; (item=Stallation::Find(NULL,i)); ++i)
		list->AddItem(item);
	list->SetInvocationMessage(new BMessage(ITEMS_INVOKED));
	
	AddChild
	(
		new MBorder
		(
			M_LABELED_BORDER,
			8,
			"Installations",
			new VGroup
			(
				new MScrollView(list,false,true),
				new HGroup
				(
					new Space(),
					new MButton("Merge",1),
					new MButton("Uninstall",2),
					0
				),
				0
			)
		),
		0
	);
}

void InstallationManager::MessageReceived(BMessage *message)
{
	if (message->what == M_BUTTON_SELECTED)
	switch (message->FindInt32(M_BUTTON_ID))
	{
		case 1: Merge();	break;
		case 2: Destall();	break;
	}
	else if (message->what == ITEMS_INVOKED)
	{
		for (int i=0; message->HasInt32("index",i); ++i)
		{
			int32 item;
			message->FindInt32("index",i,&item);
			Stallation *it = (Stallation*)list->ItemAt(item);
			
			BString path = "/tmp/";
			path << it->Name();
			BFile f(path.String(),B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
			BString sig = "application/x-vnd.Be-query";
			f.WriteAttr("BEOS:TYPE",B_MIME_TYPE,0,sig.String(),sig.Length()+1);
			
			BString qs = STALL_ATTR;
			qs.Append("==\"*").Append(it->String()).Append("*\"");
			f.WriteAttr("_trk/qrystr",B_STRING_TYPE,0,qs.String(),qs.Length()+1);
			
			const char *argv[3];
			argv[0]="/boot/beos/system/Tracker";
			argv[1]=path.String();
			argv[2]=NULL;
			
			resume_thread(load_image(2,argv,(cc**)environ));
		}
	}
	else MWindow::MessageReceived(message);
}


void InstallationManager::Destall()
{
	BAlert *alert = new BAlert
	(
		"Really?",
		"Delete files if no longer claimed by an installation?",
		"Cancel","Just clean up","Nuke 'em",
		B_WIDTH_FROM_WIDEST, B_OFFSET_SPACING, B_WARNING_ALERT
	);
	alert->SetShortcut(0,B_ESCAPE);
	int32 button = alert->Go();
	
	if (button==0) return;
	bool nuke = (button==2);
	
	int32 i=0, item;
	while ((item=list->CurrentSelection(i++)) >= 0)
	{
		Stallation *it = (Stallation*)list->RemoveItem(item);
		it->RemoveAll(nuke);
		it->Unregister();
		delete it;
	}
}

void InstallationManager::Merge()
{
	int32 item = list->CurrentSelection(0);
	if (item<0) return;
	
	Stallation *first = (Stallation*)list->ItemAt(item);
	
	while ((item=list->CurrentSelection(1)) >= 0)
	{
		Stallation *it = (Stallation*)list->RemoveItem(item);
		first->SubsumeAll(it);
		it->Unregister();
		delete it;
	}
}


bool InstallationManager::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}
