/*
Stallation version 0.7  (2000-06-28)

Written by Peter Folk <pfolk@uni.uiuc.edu>

The contents of this file are in the public domain, but
the author would appreciate it if any useful changes or
corrections were communicated back to him, for inclusion
in a future version.
*/
#include <strings.h>
#include "StallApp.h"
#include "InstallationManager.h"
#include <be/kernel/fs_info.h>
#include <be/kernel/fs_index.h>
#include <be/interface/Alert.h>
#include <be/support/String.h>
#include <be/StorageKit.h>
#include <be/interface/Alert.h>
#include "TextEntryAlert.h"


StallApp::StallApp(): BApplication("application/x-vnd.Gargoyle-Stall")
{
	gotRefs = false;
	win=NULL;
	
	LoadExpanders();
}
StallApp::~StallApp()
{
}


void StallApp::RefsReceived(BMessage *message)
{
	entry_ref ref;
	BString name;
	Stallation *inst;
	
	// Get the name of the first file, strip .zip or .tgz,
	// and suggest it as the installation name
	{
		if (message->FindRef("refs",&ref)!=B_OK) return;
		// return if there aren't really any refs
		
		// let the app know that some refs were delivered on startup
		gotRefs=true;
		
		// Get the name, sans .zip, .tgz, .tar, or .tar.gz
		name = ref.name;
		StripZip(&name);
	}
	
	
	try {
		TextEntryAlert *getname = new TextEntryAlert
		(
			"Get Stallation Name",
			"Name this installation:",
			name.String(),
			"Cancel","Create",
			false, 200, 1, B_WIDTH_FROM_WIDEST
		);
		getname->SetShortcut(0,B_ESCAPE);
		char namebuf[1024];
		if (getname->Go(namebuf,1024)!=1)
			throw "user cancelled operation";
		name = namebuf;
	
		// Get a new installation tag for these refs
		inst = new Stallation(name.String(),true);
	} catch (...) {return;}
	
	
	// Cache destination info:
	BString dbase = "/tmp/";
	dbase << inst->String() << ":";
	
	
	// Add each dropped ref to the installation
	for (int i=0; message->FindRef("refs",i,&ref)==B_OK; ++i)
	{
		inst->Install(ref,true);
		
		// Unzip, and add the resulting dir:
		BEntry entry(&ref);
		BPath path(&entry);
		BString dest = dbase;
		dest << size_t(i+1);
		if (Expand(path.Path(), dest.String()) == B_OK)
		{
			entry_ref dref;
			get_ref_for_path(dest.String(),&dref);
			inst->Install(dref,true);
		}
	}
	
	if (win) win->Add(inst);
}


void StallApp::ReadyToRun()
{
	if (gotRefs == false)
	{
		win = new InstallationManager(BRect(100,100,300,400));
		win->Show();
	} else PostMessage(B_QUIT_REQUESTED);
}


int main()
{
	// Ensure that STALL_ATTR is indexed for all indexable volumes
	dev_t dev;
	int32 i=0;
	while ((dev=next_dev(&i)) >= 0)
		fs_create_index(dev,STALL_ATTR,B_STRING_TYPE,0);
	
	StallApp s;
	s.Run();
	
	return 0;
}


static status_t LoadExpandersDir(BPath&, vector<expander>&, cc*);
static status_t LoadExpandersDir(BPath& path, vector<expander>& expanders,cc* name)
{
	status_t status;
	
	BDirectory parent(path.Path());
	if (parent.InitCheck() != B_OK) return parent.InitCheck();
	
	BDirectory dir(&parent,name);
	if (dir.InitCheck() != B_OK)
	{
		status = parent.CreateDirectory(name,&dir);
		if (status != B_OK) return status;
	}
	// now dir is the directory with my addons in it
	
	// Load extensions and expanders from dir
	entry_ref ref;
	while (dir.GetNextRef(&ref) == B_OK)
	{
		BEntry entry(&ref);
		BPath path(&entry);
		image_id addon = load_add_on(path.Path());
		if (addon<0) continue;
		
		expander symbol;
		status_t ret =
		get_image_symbol(addon,"expand",B_SYMBOL_TYPE_TEXT,(void**)&symbol);
		if (ret == B_OK)
		{
			// (new BAlert("got it","got it","great!"))->Go();
			expanders.push_back(symbol);
		}
	}
	return B_OK;
}

status_t StallApp::LoadExpanders()
{
	cc *name="Stall";
	BPath path;
	status_t status;
	
	status = find_directory(B_USER_ADDONS_DIRECTORY, &path);
	if (status != B_OK) return status;
	
	status = LoadExpandersDir(path,expanders,name);
	if (status != B_OK) return status;
	
	
	app_info info; 
	be_app->GetAppInfo(&info); 
	BEntry entry(&info.ref); 
	entry.GetPath(&path); 
	path.GetParent(&path); 
	path.Append("Expanders");
	status = LoadExpandersDir(path,expanders,name);
	
	return status;
}

void StallApp::StripZip(BString* name)
{
	int32 off = name->FindLast('.');
	if (off!=B_ERROR) name->Remove(off,name->Length()-off);
}

status_t StallApp::Expand(const char* ref, const char* dest)
{
	for (ssize_t i=expanders.size()-1; i >= 0; --i)
	{
		status_t ret = (expanders[i])(ref,dest);
		if (ret != B_ERROR)
		{
			(new BAlert("got it","got it","great!"))->Go();
			return ret;
		} else {
			(new BAlert("missed it","missed it","damn!"))->Go();
		}
	}
	return B_ERROR;
}

void StallApp::INSTALL(const entry_ref& ref, bool interact)
{
}
