/*
Stallation version 0.7  (2000-06-28)

Written by Peter Folk <pfolk@uni.uiuc.edu>

The contents of this file are in the public domain, but
the author would appreciate it if any useful changes or
corrections were communicated back to him, for inclusion
in a future version.
*/
#ifndef GARGOYLE_STALLATION_H
#define GARGOYLE_STALLATION_H
#include <vector>
#include <be/interface/ListItem.h>

typedef const char cc;
#define STALL_ATTR "Stall::id"


class Stallation: public BStringItem
{
	uint64 tag;
	char str[20];
	
  public:
	
	// Maps Installation name to ID: contains unique
	// String() -> Name() and (Name(),index) -> tag
	// mappings.  Persistent.
	
	
	Stallation(cc* name="New Installation", bool reg =true);
	// Allocate a new installation id.  If reg, Register() it.
	
	// Create an installation (unregistered unless reg==true),
	// with Name() == name and tag == t
	Stallation(cc *name, uint64 t, bool reg=false);
	
	virtual ~Stallation();
	
	// Count the number of installations with the specified name
	static size_t Count(cc* name);
	
	// Find an installation in the registry by its name and index.
	// name=NULL causes it to look by index only.  Usingname and
	// index identifes one out of many packages with the same name.
	// name==null looks up the item by index, without regard for
	// name, where the index is determined by its location in a
	// list storted by name.
	static Stallation* Find(cc* name, int index=0);
	
	// Find an installation by its package ID
	static Stallation* Find(uint64 t);
	
	
	cc*  String() const {return str;}
	cc*  Name()   const {return Text();}
	void SetName(cc* name);
	void Register();
	void Unregister();
	
	
	// Add files to this installation
	void Install(const entry_ref&, bool interact = true);
	void AddRef(const entry_ref&, bool recursive =false);
	bool HasRef(const entry_ref&) const;
	
	// Transfer files from inst to this installation
	void SubsumeRef(const entry_ref&, Stallation* inst);
	void SubsumeAll(dev_t, Stallation* inst);
	void SubsumeAll(Stallation* inst);
	
	// Remove files from this installation
	void RemoveRef(const entry_ref&, bool recurs=false, bool nuke=false);
	void RemoveAll(dev_t dev, bool nuke_if_unclaimed);
	void RemoveAll(bool nuke_if_unclaimed);
};
#endif
